"""."""

from kinematic_tracker.association.det_type import FMT, IVT


class ScoreCopy:
    def __init__(self, score_rt: FMT, creation_ids: IVT) -> None:
        self.score_rt = score_rt.copy()
        self.creation_ids = creation_ids



"""."""

import numpy as np
import pytest

from kinematic_tracker.tracker.score_copy import ScoreCopy


def test_init() -> None:
    score_rt = np.linspace(0.5, 1.0, 6).reshape(2, 3)
    creation_ids = np.linspace(0, 2, 3, dtype=int)
    sp = ScoreCopy(score_rt, creation_ids)

    assert sp.score_rt == pytest.approx(score_rt)
    assert id(sp.score_rt) != id(score_rt)
    assert sp.creation_ids == pytest.approx(creation_ids)
    assert id(sp.creation_ids) == id(creation_ids)



"""."""

import numpy as np
import pytest

from kinematic_tracker import NdKkfTracker


def test_reset(tracker1: NdKkfTracker) -> None:
    tracker1.reset()
    tracker1.advance(0, [np.array([1.0, 2.0, 3.0, 4.0, 5.0, 6.0])])
    assert len(tracker1.tracks) == 1
    ref = np.array([1.0, 0.0, 0.0, 2.0, 0.0, 0.0, 3.0, 0.0, 0.0, 4.0, 5.0, 6.0]).reshape(12, 1)
    assert tracker1.tracks[0].kkf.kalman_filter.statePost == pytest.approx(ref)



"""."""

import pytest

from kinematic_tracker.association.g_iou_scores_aligned import GIoUAux


def test_get_giou(
    giou_aux: GIoUAux,
    disjoint_xyz: GIoUAux,
    disjoint_x: GIoUAux,
    disjoint_xy: GIoUAux,
    joint_xy: GIoUAux,
) -> None:
    """."""
    assert giou_aux.get_g_iou_diff(giou_aux) == pytest.approx(1.0)
    assert giou_aux.get_g_iou_diff(joint_xy) == pytest.approx(0.3619047619047619)
    assert joint_xy.get_g_iou_diff(giou_aux) == pytest.approx(0.3619047619047619)
    assert giou_aux.get_g_iou_diff(disjoint_x) == pytest.approx(-0.42857142857142855)
    assert disjoint_x.get_g_iou_diff(giou_aux) == pytest.approx(-0.42857142857142855)
    assert giou_aux.get_g_iou_diff(disjoint_xy) == pytest.approx(-0.8095238095238095)
    assert giou_aux.get_g_iou_diff(disjoint_xyz) == pytest.approx(-0.9285714285714286)



"""."""

import numpy as np
import pytest

from kinematic_tracker import NdKkfTracker
from kinematic_tracker.tracker.score_copy import ScoreCopy


def test_with_score_copy(tracker1: NdKkfTracker) -> None:
    rv = tracker1.advance(2000_000_000, [np.linspace(1.1, 6.1, num=6)], return_score=True)
    assert isinstance(rv, ScoreCopy)
    assert rv.score_rt == pytest.approx(0.84785213)
    assert rv.score_rt.ndim == 2
    assert rv.creation_ids == pytest.approx([0])



import numpy as np
import pytest

from kinematic_tracker import NdKkfTracker


def test_get_creation_ids_one(tracker1: NdKkfTracker) -> None:
    ids = tracker1.get_creation_ids()
    assert ids == pytest.approx([0])


def test_get_creation_ids_two(tracker1: NdKkfTracker) -> None:
    det_rz = np.linspace(1.0, 12, 12).reshape(2, 6)
    tracker1.advance(2000_000_000, det_rz)
    ids = tracker1.get_creation_ids()
    assert ids == pytest.approx([0, 1])
